/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	scheduler.h
	Handles the creation and management of threads.

******************************************************************************/

#ifndef SORTIX_SCHEDULER_H
#define SORTIX_SCHEDULER_H

namespace Sortix
{
	class ProcessInfo
	{
	public:
		int lol;

	};

	enum ThreadState { INFANT, RUNNABLE, UNRUNNABLE, NOOP };
	typedef void* (*ThreadEntry)(void* Parameter);

	class ThreadInfo
	{
	public:
		ThreadInfo(ProcessInfo* Process, size_t Id, size_t* Stack, size_t StackLength);
		~ThreadInfo();

	public:
		size_t GetId() { return Id; }
		ProcessInfo* GetProcess() { return Process; }

	private:
		size_t Id;
		size_t* Stack;
		size_t StackLength;
		ProcessInfo* Process;
		ThreadState State;
		ThreadInfo** InThisList;

	public:
		uintmax_t SleepMilisecondsLeft;
		ThreadInfo* NextSleepingThread;

	public:
		void Sleep(uintmax_t Miliseconds);

	public:
		ThreadInfo* PrevThread;
		ThreadInfo* NextThread;

	public:
		void SaveRegisters(CPU::InterruptRegisters* Src);
		void LoadRegisters(CPU::InterruptRegisters* Dest);

	public:
		CPU::InterruptRegisters Registers;

	private:
		void Relink(ThreadInfo** First);

	public:
		void SetState(ThreadState NewState);
		ThreadState GetState();

	};

	namespace Scheduler
	{
		void Init();

		// Scheduling
		ThreadInfo* GetCurrentThread();
		ProcessInfo* GetCurrentProcess();
		void Switch(CPU::InterruptRegisters* R, uintmax_t TimePassed);
		SORTIX_NORETURN void MainLoop();
		void WakeSleeping(uintmax_t TimePassed);

		// Thread management
		ThreadInfo* CreateThread(ProcessInfo* Process, ThreadEntry Start, void* Parameter1 = NULL, void* Parameter2 = NULL, size_t StackSize = SIZE_MAX);
		void ExitThread(ThreadInfo* Thread, void* Result = NULL);
		
		// System Calls.
		void SysCreateThread(CPU::InterruptRegisters* R);
		void SysExitThread(CPU::InterruptRegisters* R);
		void SysSleep(CPU::InterruptRegisters* R);
		void SysUSleep(CPU::InterruptRegisters* R);
	}
}

#endif

